@@ -30,7 +30,7 @@
 
 #define DENIED "<html><head><title>libmicrohttpd demo</title></head><body>Access denied</body></html>"
 
-#define OPAQUE "11733b200778ce33060f31c9af70a870ba96ddd4"
+#define OPAQUE_STR "11733b200778ce33060f31c9af70a870ba96ddd4"
 
 static int
 ahc_echo (void *cls,
@@ -53,7 +53,7 @@ ahc_echo (void *cls,
 						 DENIED,
 						 MHD_RESPMEM_PERSISTENT);  
       ret = MHD_queue_auth_fail_response(connection, realm,
-					 OPAQUE,
+					 OPAQUE_STR,
 					 response,
 					 MHD_NO);    
       MHD_destroy_response(response);  
@@ -73,7 +73,7 @@ ahc_echo (void *cls,
       if (NULL == response) 
 	return MHD_NO;
       ret = MHD_queue_auth_fail_response(connection, realm,
-					 OPAQUE,
+					 OPAQUE_STR,
 					 response,
 					 (ret == MHD_INVALID_NONCE) ? MHD_YES : MHD_NO);  
       MHD_destroy_response(response);  
