@@ -145,7 +145,7 @@ MHD_tls_connection_handle_idle (struct MHD_Connection *connection)
     {
       /* on newly created connections we might reach here before any reply has been received */
     case MHD_TLS_CONNECTION_INIT:
-      return MHD_YES;
+      break;
       /* close connection if necessary */
     case MHD_CONNECTION_CLOSED:
       gnutls_bye (connection->tls_session, GNUTLS_SHUT_RDWR);
@@ -156,7 +156,11 @@ MHD_tls_connection_handle_idle (struct MHD_Connection *connection)
 	return MHD_YES;
       return MHD_connection_handle_idle (connection);
     }
+#if EPOLL_SUPPORT   
+  return MHD_connection_epoll_update_ (connection);
+#else
   return MHD_YES;
+#endif
 }
 
 
