@@ -452,7 +452,8 @@ testStopRace (int poll_flag)
     struct MHD_Daemon *d;
     
     d = MHD_start_daemon (MHD_USE_THREAD_PER_CONNECTION | MHD_USE_DEBUG | poll_flag,
-                         1081, NULL, NULL, &ahc_echo, "GET", MHD_OPTION_END);
+                         1081, NULL, NULL, &ahc_echo, "GET",
+                         MHD_OPTION_CONNECTION_TIMEOUT, 5, MHD_OPTION_END);
     if (d == NULL)
        return 16;
     
